"""
Some constants related to specific Prime 1 items that the logic needs to deal with.
"""

ARTIFACT_MODEL = [29 + i for i in range(12)]
ARTIFACT_ITEMS = [29 + i for i in range(12)]
ARTIFACT_NAMES = [
    "Artifact of Truth",
    "Artifact of Strength",
    "Artifact of Elder",
    "Artifact of Wild",
    "Artifact of Lifegiver",
    "Artifact of Warrior",
    "Artifact of Chozo",
    "Artifact of Nature",
    "Artifact of Sun",
    "Artifact of World",
    "Artifact of Spirit",
    "Artifact of Newborn"
]
