"""
Some constants related to specific Echoes items that the logic needs to deal with.
"""

DARK_TEMPLE_KEY_MODEL = 37
DARK_TEMPLE_KEY_NAMES = [
    "Dark Agon Key {0}",
    "Dark Torvus Key {0}",
    "Ing Hive Key {0}"
]
DARK_TEMPLE_KEY_ITEMS = [
    [32, 33, 34, ],
    [35, 36, 37, ],
    [38, 39, 40, ],
]
SKY_TEMPLE_KEY_MODEL = 38
SKY_TEMPLE_KEY_ITEMS = [
    29,
    30,
    31,
    101,
    102,
    103,
    104,
    105,
    106,
]
USELESS_PICKUP_MODEL = 30
USELESS_PICKUP_ITEM = 107
